@@ -87,9 +87,11 @@ Rcpp::List cICS_L(arma::vec data,
 
   // initialize results objects
   arma::mat result_clust(niter - nburn, n);
-  std::list<arma::vec> result_mu;
+  arma::field<arma::vec> result_mu(niter - nburn);
+  arma::field<arma::vec> result_probs(niter - nburn);
+  // std::list<arma::vec> result_mu;
+  // std::list<arma::vec> result_probs;
   arma::vec result_s2(niter - nburn);
-  std::list<arma::vec> result_probs;
   arma::mat result_dens(niter - nburn, grid.n_elem);
   result_dens.fill(0);
 
@@ -160,6 +162,7 @@ Rcpp::List cICS_L(arma::vec data,
     arma::vec index = arma::linspace(0, nkpt - 1, nkpt);
     arma::vec probjoin = arma::join_cols(ptilde.elem(arma::find(index < nkpt - 1)),
                                          ptilde[nkpt - 1] * freqtemp / napprox);
+    probjoin = probjoin / sum(probjoin);
 
     // update cluster allocations
     clust_update_ICS_L(data,
@@ -172,7 +175,6 @@ Rcpp::List cICS_L(arma::vec data,
 
     // if the burn-in phase is complete
     if(iter >= nburn){
-      result_probs.push_back(probjoin);
       if(out_dens){
         dens = eval_density_L(grid,
                               mujoin,
@@ -182,15 +184,29 @@ Rcpp::List cICS_L(arma::vec data,
       }
     }
 
-    // clean parameter objects
-    para_clean_ICS_L(mu,
-                     clust);
+    if(out_param){
+      // clean parameter objects
+      para_clean_ICS_L_export(mu,
+                              mujoin,
+                              probjoin,
+                              clust);
 
-    // if the burn-in phase is complete
-    if(iter >= nburn){
-      result_mu.push_back(mu);
-      result_s2(iter - nburn) = s2;
-      result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_probs(iter - nburn) = probjoin;
+        result_mu(iter - nburn) = mujoin;
+        result_s2(iter - nburn) = s2;
+        result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      }
+    } else {
+      // clean parameter objects
+      para_clean_ICS_L(mu,
+                       clust);
+
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      }
     }
 
     if(print_message){
@@ -293,9 +309,12 @@ Rcpp::List cICS(arma::vec data,
 
   // initialize results objects
   arma::mat result_clust(niter - nburn, n);
-  std::list<arma::vec> result_mu;
-  std::list<arma::vec> result_s2;
-  std::list<arma::vec> result_probs;
+  arma::field<arma::vec> result_mu(niter - nburn);
+  arma::field<arma::vec> result_s2(niter - nburn);
+  arma::field<arma::vec> result_probs(niter - nburn);
+  // std::list<arma::vec> result_mu;
+  // std::list<arma::vec> result_s2;
+  // std::list<arma::vec> result_probs;
   arma::mat result_dens(niter - nburn, grid.n_elem);
   result_dens.fill(0);
 
@@ -382,6 +401,7 @@ Rcpp::List cICS(arma::vec data,
     arma::vec index = arma::linspace(0, nkpt - 1, nkpt);
     arma::vec probjoin = arma::join_cols(ptilde.elem(arma::find(index < nkpt - 1)),
                                          ptilde[nkpt - 1] * freqtemp / napprox);
+    probjoin = probjoin / sum(probjoin);
 
     // update cluster allocations
     clust_update_ICS(data,
@@ -395,7 +415,6 @@ Rcpp::List cICS(arma::vec data,
 
     // if the burn-in phase is complete
     if(iter >= nburn){
-      result_probs.push_back(probjoin);
       if(out_dens){
         dens = eval_density(grid,
                             mujoin,
@@ -405,34 +424,33 @@ Rcpp::List cICS(arma::vec data,
       }
     }
 
-    // clean parameter objects
-    para_clean_ICS(mu,
-                   s2,
-                   clust);
-
-    // if the burn-in phase is complete
-    if(iter >= nburn){
-      result_mu.push_back(mu);
-      result_s2.push_back(s2);
-      result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
-    }
-
-    ///////////////////////////////////////////////////////////////////////
-    ///////////////////////////////////////////////////////////////////////
-    // temp////////////////////////////////////////////////////////////////
-    if(iter >= nburn){
-      arma::vec tab_freq = freq_vec(clust) - discount;
-      if(out_dens){
-        dens = eval_density(grid,
-                            mu,
+    if(out_param){
+      // clean parameter objects
+      para_clean_ICS_export(mu,
                             s2,
-                            tab_freq);
-        result_dens_dev.row(iter - nburn) = arma::trans(dens);
+                            mujoin,
+                            s2join,
+                            probjoin,
+                            clust);
+
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_mu(iter - nburn) = mujoin;
+        result_s2(iter - nburn) = s2join;
+        result_probs(iter - nburn) = probjoin;
+        result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      }
+    } else {
+      // clean parameter objects
+      para_clean_ICS(mu,
+                     s2,
+                     clust);
+
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
       }
     }
-    ////////////////////////////////////////////////////////////////////////
-    ////////////////////////////////////////////////////////////////////////
-    ////////////////////////////////////////////////////////////////////////
 
     if(print_message){
       // print the current completed work
@@ -462,15 +480,6 @@ Rcpp::List cICS(arma::vec data,
     resu["clust"]  = result_clust;
     resu["time"]   = double(end_s-start_s)/CLOCKS_PER_SEC;
   }
-
-  ///////////////////////////////////////////////////////////////////////
-  ///////////////////////////////////////////////////////////////////////
-  // temp////////////////////////////////////////////////////////////////
-  resu["tdns"] = result_dens_dev;
-  ///////////////////////////////////////////////////////////////////////
-  ///////////////////////////////////////////////////////////////////////
-  ///////////////////////////////////////////////////////////////////////
-
   return resu;
 }
 
@@ -542,9 +551,10 @@ Rcpp::List cICS_mv_L(arma::mat data,
 
   // initialize results objects
   arma::mat result_clust(niter - nburn, n);
-  std::list<arma::mat> result_mu;
+  arma::field<arma::mat> result_mu(niter - nburn);
   arma::cube result_s2(d,d,niter - nburn);
-  std::list<arma::vec> result_probs;
+  arma::field<arma::vec> result_probs(niter - nburn);
+
   arma::mat result_dens(grid.n_rows,1);
   if(!light_dens){
     result_dens.resize(niter - nburn, grid.n_rows);
@@ -617,6 +627,7 @@ Rcpp::List cICS_mv_L(arma::mat data,
     arma::vec index = arma::linspace(0, nkpt - 1, nkpt);
     arma::vec probjoin = arma::join_cols(ptilde.elem(arma::find(index < nkpt - 1)),
                                          ptilde[nkpt - 1] * freqtemp / napprox);
+    probjoin = probjoin / sum(probjoin);
 
     // update cluster allocations
     clust_update_ICS_mv_L(data,
@@ -629,7 +640,6 @@ Rcpp::List cICS_mv_L(arma::mat data,
 
     // if the burn-in phase is complete
     if(iter >= nburn){
-      result_probs.push_back(probjoin);
       if(out_dens){
         dens = eval_density_mv_L(grid,
                                  mujoin,
@@ -643,15 +653,29 @@ Rcpp::List cICS_mv_L(arma::mat data,
       }
     }
 
-    // clean parameter objects
-    para_clean_ICS_mv_L(mu,
-                        clust);
+    if(out_param){
+      // clean parameter objects
+      para_clean_ICS_mv_L_export(mu,
+                                 mujoin,
+                                 probjoin,
+                                 clust);
 
-    // if the burn-in phase is complete
-    if(iter >= nburn){
-      result_mu.push_back(mu);
-      result_s2.slice(iter - nburn) = s2;
-      result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_probs(iter - nburn) = probjoin;
+        result_mu(iter - nburn) = mujoin;
+        result_s2.slice(iter - nburn) = s2;
+        result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      }
+    } else {
+      // clean parameter objects
+      para_clean_ICS_mv_L(mu,
+                          clust);
+
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      }
     }
 
     if(print_message){
@@ -766,9 +790,9 @@ Rcpp::List cICS_mv(arma::mat data,
 
   // initialize results objects
   arma::mat result_clust(niter - nburn, n);
-  std::list<arma::mat> result_mu;
-  std::list<arma::cube> result_s2;
-  std::list<arma::vec> result_probs;
+  arma::field<arma::mat> result_mu(niter - nburn);
+  arma::field<arma::cube> result_s2(niter - nburn);
+  arma::field<arma::vec> result_probs(niter - nburn);
   arma::mat result_dens(grid.n_rows,1);
   if(!light_dens){
     result_dens.resize(niter - nburn, grid.n_rows);
@@ -793,17 +817,17 @@ Rcpp::List cICS_mv(arma::mat data,
   s2temp.fill(0);
   freqtemp.fill(1);
 
+  //-*-temp
+  double new_clust;
+  arma::vec clust_news(niter - nburn);
+
   // time quantities
   int start_s = clock();
   int current_s;
 
-  // TEMP TEMP TEMP TEMP
-  arma::mat times_in(niter, 6);
   // strarting loop
   for(arma::uword iter = 0; iter < niter; iter++){
 
-    int in_start_s = clock();
-
     // acceleration step
     accelerate_ICS_mv(data,
                       mu,
@@ -813,8 +837,6 @@ Rcpp::List cICS_mv(arma::mat data,
                       k0,
                       S0,
                       n0);
-    times_in(iter, 0) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
-    in_start_s = clock();
 
     if(hyper){
       hyper_accelerate_ICS_mv_LS(mu,
@@ -830,16 +852,12 @@ Rcpp::List cICS_mv(arma::mat data,
                                  theta1,
                                  Theta1);
     }
-    times_in(iter, 1) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
-    in_start_s = clock();
 
     // sample the probability vector
     // from Dirichlet distribution
     arma::vec temp_freq = freq_vec(clust);
     ptilde = rdirich_mass(temp_freq - discount, strength + discount * temp_freq.n_elem);
 
-    times_in(iter, 2) = double(clock() - in_start_s ) / CLOCKS_PER_SEC ;
-    in_start_s = clock();
 
     // simulate the required values
     simu_trunc_PY_mv(mutemp,
@@ -852,7 +870,6 @@ Rcpp::List cICS_mv(arma::mat data,
                      n0,
                      napprox,
                      discount);
-    times_in(iter, 3) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
 
     // joint the existent parameters with
     // the simulated new ones
@@ -862,22 +879,22 @@ Rcpp::List cICS_mv(arma::mat data,
     arma::vec index = arma::linspace(0, nkpt - 1, nkpt);
     arma::vec probjoin = arma::join_cols(ptilde.elem(arma::find(index < nkpt - 1)),
                                          ptilde[nkpt - 1] * freqtemp / napprox);
-    in_start_s = clock();
+    probjoin = probjoin / sum(probjoin);
 
     // update cluster allocations
     clust_update_ICS_mv(data,
                         mujoin,
                         s2join,
                         probjoin,
-                        clust);
-    times_in(iter, 4) = double(clock() - in_start_s) / CLOCKS_PER_SEC ;
+                        clust,
+                        //-*-temp
+                        new_clust);
 
     mu = mujoin;
     s2 = s2join;
 
     // if the burn-in phase is complete
     if(iter >= nburn){
-      result_probs.push_back(probjoin);
       if(out_dens){
         dens = eval_density_mv(grid,
                                mujoin,
@@ -889,18 +906,36 @@ Rcpp::List cICS_mv(arma::mat data,
           result_dens.row(iter - nburn) = arma::trans(dens);
         }
       }
+      //-*-temp
+      clust_news(iter - nburn) = new_clust;
     }
 
-    // clean parameter objects
-    para_clean_ICS_mv(mu,
-                      s2,
-                      clust);
+    if(out_param){
+      // clean parameter objects
+      para_clean_ICS_mv_export(mu,
+                               s2,
+                               mujoin,
+                               s2join,
+                               probjoin,
+                               clust);
 
-    // if the burn-in phase is complete
-    if(iter >= nburn){
-      result_mu.push_back(mu);
-      result_s2.push_back(s2);
-      result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_mu(iter - nburn) = mujoin;
+        result_s2(iter - nburn) = s2join;
+        result_probs(iter - nburn) = probjoin;
+        result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      }
+    } else {
+      // clean parameter objects
+      para_clean_ICS_mv(mu,
+                        s2,
+                        clust);
+
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      }
     }
 
     if(print_message){
@@ -940,8 +975,8 @@ Rcpp::List cICS_mv(arma::mat data,
     resu["clust"]  = result_clust;
     resu["time"]   = double(end_s-start_s)/CLOCKS_PER_SEC;
   }
-  // TEMP TEMP TEMP
-  resu["wvals"] = times_in;
+  //-*-temp
+  resu["wvals"] = clust_news;
   return resu;
 }
 
@@ -1017,9 +1052,9 @@ Rcpp::List cICS_mv_P(arma::mat data,
 
   // initialize results objects
   arma::mat result_clust(niter - nburn, n);
-  std::list<arma::mat> result_mu;
-  std::list<arma::mat> result_s2;
-  std::list<arma::vec> result_probs;
+  arma::field<arma::mat> result_mu(niter - nburn);
+  arma::field<arma::mat> result_s2(niter - nburn);
+  arma::field<arma::vec> result_probs(niter - nburn);
   arma::mat result_dens(grid.n_rows,1);
   if(!light_dens){
     result_dens.resize(niter - nburn, grid.n_rows);
@@ -1102,6 +1137,7 @@ Rcpp::List cICS_mv_P(arma::mat data,
     arma::vec index = arma::linspace(0, nkpt - 1, nkpt);
     arma::vec probjoin = arma::join_cols(ptilde.elem(arma::find(index < nkpt - 1)),
                                          ptilde[nkpt - 1] * freqtemp / napprox);
+    probjoin = probjoin / sum(probjoin);
 
     // update cluster allocations
     clust_update_ICS_mv_P(data,
@@ -1115,7 +1151,6 @@ Rcpp::List cICS_mv_P(arma::mat data,
 
     // if the burn-in phase is complete
     if(iter >= nburn){
-      result_probs.push_back(probjoin);
       if(out_dens){
         dens = eval_density_mv_P(grid,
                                  mujoin,
@@ -1130,16 +1165,32 @@ Rcpp::List cICS_mv_P(arma::mat data,
       }
     }
 
-    // clean parameter objects
-    para_clean_ICS_mv_P(mu,
-                        s2,
-                        clust);
+    if(out_param){
+      // clean parameter objects
+      para_clean_ICS_mv_P_export(mu,
+                                 s2,
+                                 mujoin,
+                                 s2join,
+                                 probjoin,
+                                 clust);
 
-    // if the burn-in phase is complete
-    if(iter >= nburn){
-      result_mu.push_back(mu);
-      result_s2.push_back(s2);
-      result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_probs(iter - nburn) = probjoin;
+        result_mu(iter - nburn) = mujoin;
+        result_s2(iter - nburn) = s2join;
+        result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      }
+    } else {
+      // clean parameter objects
+      para_clean_ICS_mv_P(mu,
+                          s2,
+                          clust);
+
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_clust.row(iter - nburn) = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      }
     }
 
     if(print_message){
@@ -1257,9 +1308,9 @@ Rcpp::List cICS_mv_MKR(arma::vec y,
   // initialize results objects
   arma::mat result_clust(niter - nburn, n);
   arma::mat result_beta(niter - nburn, d);
-  std::list<arma::mat> result_beta_all;
-  std::list<arma::vec> result_sigma2;
-  std::list<arma::vec> result_probs;
+  arma::field<arma::mat> result_beta_all(niter - nburn);
+  arma::field<arma::vec> result_sigma2(niter - nburn);
+  arma::field<arma::vec> result_probs(niter - nburn);
 
   // initialize the result densities objects (response and covs)
   arma::cube result_dens(grid_response.n_rows, grid_covs.n_rows, 1);
@@ -1352,6 +1403,7 @@ Rcpp::List cICS_mv_MKR(arma::vec y,
     arma::vec index = arma::linspace(0, nkpt - 1, nkpt);
     probjoin = arma::join_cols(ptilde.elem(arma::find(index < nkpt - 1)),
                                          ptilde[nkpt - 1] * freqtemp / napprox);
+    probjoin = probjoin / sum(probjoin);
 
     // update cluster allocations
     clust_update_ICS_mv_MRK(y,
@@ -1366,8 +1418,6 @@ Rcpp::List cICS_mv_MKR(arma::vec y,
 
     // if the burn-in phase is complete
     if(iter >= nburn){
-      result_probs.push_back(probjoin);
-
       if(out_dens){
         dens = eval_density_mv_MKR(grid_covs,
                                    grid_response,
@@ -1383,16 +1433,32 @@ Rcpp::List cICS_mv_MKR(arma::vec y,
       }
     }
 
-    // clean parameter objects
-    para_clean_ICS_mv_MRK(beta,
-                          sigma2,
-                          clust);
+    if(out_param){
+      // clean parameter objects
+      para_clean_ICS_mv_MRK_export(beta,
+                                   sigma2,
+                                   betajoin,
+                                   sigma2join,
+                                   probjoin,
+                                   clust);
 
-    // if the burn-in phase is complete
-    if(iter >= nburn){
-      result_beta_all.push_back(beta);
-      result_sigma2.push_back(sigma2);
-      result_clust.row(iter - nburn)  = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_beta_all(iter - nburn) = beta;
+        result_sigma2(iter - nburn) = sigma2;
+        result_probs(iter - nburn) = probjoin;
+        result_clust.row(iter - nburn)  = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      }
+    } else {
+      // clean parameter objects
+      para_clean_ICS_mv_MRK(beta,
+                            sigma2,
+                            clust);
+
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_clust.row(iter - nburn)  = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      }
     }
 
     if(print_message){
@@ -1506,9 +1572,10 @@ Rcpp::List cICS_mv_MKR_L(arma::vec y,
   // initialize results objects
   arma::mat result_clust(niter - nburn, n);
   arma::mat result_beta(niter - nburn, d);
-  std::list<arma::mat> result_beta_all;
+
+  arma::field<arma::mat> result_beta_all(niter - nburn);
   arma::vec result_sigma2(niter - nburn);
-  std::list<arma::vec> result_probs;
+  arma::field<arma::vec> result_probs(niter - nburn);
 
   // initialize the result densities objects (response and covs)
   arma::cube result_dens(grid_response.n_rows, grid_covs.n_rows, 1);
@@ -1591,6 +1658,7 @@ Rcpp::List cICS_mv_MKR_L(arma::vec y,
     arma::vec index = arma::linspace(0, nkpt - 1, nkpt);
     probjoin = arma::join_cols(ptilde.elem(arma::find(index < nkpt - 1)),
                                ptilde[nkpt - 1] * freqtemp / napprox);
+    probjoin = probjoin / sum(probjoin);
 
     // update cluster allocations
     clust_update_ICS_mv_MRK_L(y,
@@ -1604,8 +1672,6 @@ Rcpp::List cICS_mv_MKR_L(arma::vec y,
 
     // if the burn-in phase is complete
     if(iter >= nburn){
-      result_probs.push_back(probjoin);
-
       if(out_dens){
         dens = eval_density_mv_MKR_L(grid_covs,
                                      grid_response,
@@ -1621,15 +1687,29 @@ Rcpp::List cICS_mv_MKR_L(arma::vec y,
       }
     }
 
-    // clean parameter objects
-    para_clean_ICS_mv_MRK_L(beta,
-                            clust);
+    if(out_param){
+      // clean parameter objects
+      para_clean_ICS_mv_MRK_L_export(beta,
+                                     betajoin,
+                                     probjoin,
+                                     clust);
 
-    // if the burn-in phase is complete
-    if(iter >= nburn){
-      result_beta_all.push_back(beta);
-      result_sigma2(iter - nburn) = sigma2;
-      result_clust.row(iter - nburn)  = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_beta_all(iter - nburn) = beta;
+        result_sigma2(iter - nburn) = sigma2;
+        result_probs(iter - nburn) = probjoin;
+        result_clust.row(iter - nburn)  = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      }
+    } else {
+      // clean parameter objects
+      para_clean_ICS_mv_MRK_L(beta,
+                              clust);
+
+      // if the burn-in phase is complete
+      if(iter >= nburn){
+        result_clust.row(iter - nburn)  = arma::trans(arma::conv_to<arma::vec>::from(clust));
+      }
     }
 
     if(print_message){
